// BlockingSocket.h: interface for the CBlockingSocket class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BLOCKINGSOCKET_H__E9DB1167_6525_42B9_9A35_826277C9F739__INCLUDED_)
#define AFX_BLOCKINGSOCKET_H__E9DB1167_6525_42B9_9A35_826277C9F739__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

struct SocketException
{
	int err;
};

class CBlockingSocket  
{
public:
	CBlockingSocket()
	{
		m_readbuffer = new char[1024];
		m_hSocket = 0;
		m_dwReadUsed = 0;
	}

	~CBlockingSocket()
	{
		if(m_hSocket)
			closesocket(m_hSocket);
		delete [] m_readbuffer;
	}
	void Create(DWORD type = SOCK_STREAM)
	{
		ASSERT(m_hSocket == NULL);
		m_hSocket = socket(AF_INET, type,0);

		if(!m_hSocket)
		{
			SocketException e;
			e.err = WSAGetLastError();
			throw e;
		}
	}
	void Connect(const char* addr, USHORT port)
	{
		Connect(*(ULONG*) gethostbyname(addr)->h_addr_list[0], port);
	}
	void Connect(DWORD IPv4, USHORT port)
	{
		ASSERT(m_hSocket != NULL);

		sockaddr_in addr;
		addr.sin_port = htons(port);
		addr.sin_family = AF_INET;
		addr.sin_addr.s_addr = IPv4;
		if (connect(m_hSocket, (sockaddr*) &addr, sizeof (addr)) == SOCKET_ERROR)
		{
			SocketException e;
			e.err = WSAGetLastError();
			throw e;
		}

	}
	bool Close()
	{
		if(m_hSocket)
		{
			closesocket(m_hSocket);
			m_hSocket = NULL;
		}
	}
	void Write(const char* buffer, DWORD size)
	{
		int sent = 0;
		while(sent < size)
		{
			int thistime = send(m_hSocket, buffer+sent, size-sent, 0);
			if(thistime == SOCKET_ERROR)
			{
				SocketException e;
				e.err = WSAGetLastError();
				throw e;
			}
			else
				sent += thistime;
		}
	}
	int Read(char* buffer, int max)
	{
		int read = 0;
		
		
		while(read < max)
		{
			int thistime = recv(m_hSocket, buffer + read, max - read, 0);
		//	if(thistime == SOCKET_ERROR)
		//	{
		//		SocketException e;
		//		e.err = WSAGetLastError();
		//		throw e;
		//	}
			if(thistime != -1)
				read += thistime;

			if(thistime <= 0)
				break;
		}
		return read;
	}
	CString ReadLine()
	{
		CString out;
		while(true)
		{
			if(!m_dwReadUsed)
				m_dwReadUsed += Read(m_readbuffer, 1024);
			if(!m_dwReadUsed)
				return out;
			void* pos = memchr(m_readbuffer, '\r', m_dwReadUsed);
			if(!pos)
			{
				out += CString(m_readbuffer, m_dwReadUsed);
				m_dwReadUsed = 0;
			}
			else
			{
				out += CString(m_readbuffer, ((char*)pos) - m_readbuffer);
				memmove(m_readbuffer, ((char*)pos) + 2, m_dwReadUsed-(((char*)pos)-m_readbuffer));
				m_dwReadUsed -= ((char*)pos) - m_readbuffer + 2;
				break;
			}
		}
		return out;
	}
	operator SOCKET ()
	{
		return m_hSocket;
	}
private:
	SOCKET m_hSocket;
	char* m_readbuffer;
	DWORD m_dwReadUsed;
};

#endif // !defined(AFX_BLOCKINGSOCKET_H__E9DB1167_6525_42B9_9A35_826277C9F739__INCLUDED_)
